@@ -40,44 +40,39 @@
 
 typedef struct
 {
-  int 		x;
-  int 		y;
-
-  unsigned char key_codes[512];
-  unsigned char sensor_codes[32];
-} BRLTTY_DEVICE_DATA;
+    gint   x;
+    gint   y;
+    guchar key_codes[512];
+    guchar sensor_codes[32];
+} BRLTTYDeviceData;
 
 /* Globals */
 static GIOChannel 		*gioch = NULL;
-static BRL_DEV_CALLBACK		ClientCallback = NULL;
-static BRLTTY_DEVICE_DATA 	dd;
+static BRLDevCallback		client_callback = NULL;
+static BRLTTYDeviceData 	dd;
 
 /* Functions */
 
-/* function brltty_brl_send_dots
- *
- * Send array of dots of length count to brlapi.  Argument blocking is
- * ignored.
- */
-int
-brltty_brl_send_dots (unsigned char 	*dots, 
-		      short 		count, 
-		      short 		blocking)
+
+/* Send array of dots of length count to brlapi.  Argument blocking is ignored. */
+gint
+brltty_brl_send_dots (guchar 	*dots, 
+		      gshort 	count, 
+		      gshort 	blocking)
 {
-    int 		i, 
+    gint 		i, 
 			len = dd.x * dd.y;
-    unsigned char 	sendbuff[256];
+    guchar 	        sendbuff[256];
   
-			
     if (count > len) 
 	return 0;
 
-  /* Gnopernicus's idea of how to arrange braille dots
-   * is slightly different from BRLTTY's representation.
-   */
+    /* Gnopernicus's idea of how to arrange braille dots
+    * is slightly different from BRLTTY's representation.
+    */
     for (i = 0; i < count; i++) 
     {
-	unsigned char val = 0;
+	guchar val = 0;
 	
 	if (dots[i] & 0x01 ) 
 	    val = val|B1;
@@ -117,12 +112,14 @@ brltty_brl_close_device ()
 }
 
 static gboolean
-brltty_brl_glib_cb (GIOChannel *source, GIOCondition condition, gpointer data)
+brltty_brl_glib_cb (GIOChannel   *source, 
+		    GIOCondition condition, 
+		    gpointer     data)
 {
     brl_keycode_t	keypress;
 
-    BRAILLE_EVENT_CODE 	bec;
-    BRAILLE_EVENT_DATA 	bed;
+    BRLEventCode 	bec;
+    BRLEventData 	bed;
 
     while (brlapi_readCommand (0, &keypress) == 1) 
     {
@@ -131,58 +128,58 @@ brltty_brl_glib_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 	{
 	    case CMD_LNUP:
     		sprintf(&dd.key_codes[0], "DK00");
-    		bec = bec_key_codes;
-    		bed.KeyCodes = dd.key_codes;						
-    		ClientCallback (bec, &bed);
-    		break;
+    		bec = BRL_EVCODE_KEY_CODES;
+    		bed.key_codes = dd.key_codes;						
+    		client_callback (bec, &bed);
+    	    break;
       
 	    case CMD_HOME:
     		sprintf(&dd.key_codes[0], "DK01");
-    		bec = bec_key_codes;
-    		bed.KeyCodes = dd.key_codes;						
-    		ClientCallback (bec, &bed);
-    		break;
+    		bec = BRL_EVCODE_KEY_CODES;
+    		bed.key_codes = dd.key_codes;						
+    		client_callback (bec, &bed);
+    	    break;
 
 	    case CMD_LNDN:
     		sprintf(&dd.key_codes[0], "DK02");
-    		bec = bec_key_codes;
-    		bed.KeyCodes = dd.key_codes;						
-    		ClientCallback (bec, &bed);
-    		break;
+    		bec = BRL_EVCODE_KEY_CODES;
+    		bed.key_codes = dd.key_codes;						
+    		client_callback (bec, &bed);
+    	    break;
       
 	    case CMD_FWINLT:
     		sprintf(&dd.key_codes[0], "DK03");
-    		bec = bec_key_codes;
-    		bed.KeyCodes = dd.key_codes;						
-    		ClientCallback (bec, &bed);
-    		break;
+    		bec = BRL_EVCODE_KEY_CODES;
+    		bed.key_codes = dd.key_codes;						
+    		client_callback (bec, &bed);
+    	    break;
       
 	    case CMD_FWINRT:
     		sprintf(&dd.key_codes[0], "DK05");
-        	bec = bec_key_codes;
-    		bed.KeyCodes = dd.key_codes;						
-    		ClientCallback (bec, &bed);
-    		break;
+        	bec = BRL_EVCODE_KEY_CODES;
+    		bed.key_codes = dd.key_codes;						
+    		client_callback (bec, &bed);
+    	    break;
       
     	    /* TBD: Default action (DK01DK02) and repeat last */
 	    default: 
 	    {
-    		int key = keypress & VAL_BLK_MASK;
-    		int arg = keypress & VAL_ARG_MASK;
+    		gint key = keypress & VAL_BLK_MASK;
+    		gint arg = keypress & VAL_ARG_MASK;
 
     		switch (key) 
     		{
     		    case CR_ROUTE:
 			sprintf(&dd.sensor_codes[0], "HMS%02d", arg);
-			bec = bec_sensor;
-			bed.Sensor.SensorCodes = dd.sensor_codes;
-			ClientCallback (bec, &bed);
-			break;
+			bec = BRL_EVCODE_SENSOR;
+			bed.sensor.sensor_codes = dd.sensor_codes;
+			client_callback (bec, &bed);
+		    break;
 		    default:
 			fprintf(stderr,"BRLTTY command code not bound to Gnopernicus key code: 0X%04X\n", keypress);
-			break;
+		    break;
     		}
-    		break;
+    	    break;
 	    }
 	}
     }
@@ -190,15 +187,15 @@ brltty_brl_glib_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 }
 
 static void
-ignoreBlock (int block)
+ignore_block (gint block)
 {
     brlapi_ignoreKeys(block, block|VAL_ARG_MASK);
 }
 
 static void
-ignoreInput (int block)
+ignore_input (gint block)
 {
-    static const int flags[] = {
+    static const gint flags[] = {
                 0 |         0 |        0 |           0,
                 0 |         0 |        0 | VPC_CONTROL,
                 0 |         0 | VPC_META |           0,
@@ -216,21 +213,21 @@ ignoreInput (int block)
         VPC_SHIFT | VPC_UPPER | VPC_META |           0,
         VPC_SHIFT | VPC_UPPER | VPC_META | VPC_CONTROL
     };
-    const int *flag = flags + (sizeof(flags) / sizeof(*flag));
+    const gint *flag = flags + (sizeof(flags) / sizeof(*flag));
     do {
-        ignoreBlock(block | *--flag);
+        ignore_block(block | *--flag);
     } while (*flag);
 }
 
-int
-brltty_brl_open_device (char* 			DeviceName, 
-			short 			Port,
-			BRL_DEV_CALLBACK 	DeviceCallback, 
-			BRL_DEVICE 		*Device)
+gint
+brltty_brl_open_device (gchar* 			device_name, 
+			gshort 			port,
+			BRLDevCallback 	        device_callback, 
+			BRLDevice 		*device)
 {
-    int fd;
+    gint fd;
 
-    if ( (fd = brlapi_initializeConnection (NULL, NULL) ) < 0) 
+    if ((fd = brlapi_initializeConnection (NULL, NULL) ) < 0) 
     {
 	fprintf(stderr, "Error opening brlapi connection");
 	return 0;
@@ -244,22 +241,22 @@ brltty_brl_open_device (char* 			DeviceName,
 
     fprintf(stderr, "BrlAPI detected a %dx%d display\n", dd.x, dd.y);
 
-    Device->CellCount = dd.x * dd.y;
-    Device->DisplayCount = 1; /* No status cells implemented yet */
+    device->cell_count = dd.x * dd.y;
+    device->display_count = 1; /* No status cells implemented yet */
 
-    Device->Displays[0].StartCell 	= 0;
-    Device->Displays[0].Width 		= dd.x;
-    Device->Displays[0].Type 		= bdt_main;
+    device->displays[0].start_cell 	= 0;
+    device->displays[0].width 		= dd.x;
+    device->displays[0].type 		= BRL_DISP_MAIN;
 
     /* fill device functions for the upper level */
-    Device->send_dots = brltty_brl_send_dots;
-    Device->close_device = brltty_brl_close_device;
+    device->send_dots = brltty_brl_send_dots;
+    device->close_device = brltty_brl_close_device;
 	
     /* Setup glib polling for the socket fd of brlapi */
     gioch = g_io_channel_unix_new (fd);
     g_io_add_watch (gioch, G_IO_IN | G_IO_PRI, brltty_brl_glib_cb, NULL);
 
-    ClientCallback = DeviceCallback;
+    client_callback = device_callback;
 
     /* TODO: Write a function which reliably determines the TTY
      where the X server is running.  
@@ -269,10 +266,9 @@ brltty_brl_open_device (char* 			DeviceName,
     */
     brlapi_getTty (0, BRLCOMMANDS, NULL);
 
-    ignoreInput(VAL_PASSCHAR);
-    ignoreInput(VAL_PASSDOTS);
-    ignoreBlock(VAL_PASSKEY);
+    ignore_input(VAL_PASSCHAR);
+    ignore_input(VAL_PASSDOTS);
+    ignore_block(VAL_PASSKEY);
 
     return 1;
 }
-
