@@ -335,11 +335,11 @@ magconf_load_zoomer_from_schema (const gchar *schema,
 								MAGNIFIER_PANNING, 	
 								DEFAULT_MAGNIFIER_PANNING);
 
-    magnifier->zoomfactorx	= gnopiconf_get_int_from_section_with_default (
+    magnifier->zoomfactorx	= gnopiconf_get_double_from_section_with_default (
 								path, 	
 								MAGNIFIER_ZOOM_FACTOR_X, 	
 								DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY);
-    magnifier->zoomfactory	= gnopiconf_get_int_from_section_with_default (
+    magnifier->zoomfactory	= gnopiconf_get_double_from_section_with_default (
 								path, 	
 								MAGNIFIER_ZOOM_FACTOR_Y, 	
 								DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY);
@@ -438,6 +438,8 @@ magconf_set_setting_in_schema (	const gchar *id,
 			    CONFIG_PATH MAGNIFIER_PATH, 
 			    schema, 
 			    id);    
+    g_message ("setting gconf key %s", key);
+
     
     switch (data_type)
     {
@@ -451,6 +453,8 @@ magconf_set_setting_in_schema (	const gchar *id,
 	    gnopiconf_set_bool_in_section (*((gboolean*)data), path, key);
 	    break;
 	case GCONF_VALUE_FLOAT:
+	    gnopiconf_set_double_in_section (*((gdouble*)data), path, key);
+	    break;
 	case GCONF_VALUE_LIST:
 	default:
 	    break;
@@ -518,6 +522,11 @@ magconf_save_item_in_schema (const gchar *schema,
 						path, 	
 						key);
 	    break;
+	case GCONF_VALUE_FLOAT:
+		gnopiconf_set_double_in_section (*((gdouble*)data), 		
+						 path, 	
+						 key);
+	    break;
 	default:
 	 break;
     }
@@ -566,10 +575,10 @@ magconf_save_zoomer_in_schema (const gchar *schema,
 				    MAGNIFIER_PANNING);
 
 			
-    gnopiconf_set_int_in_section (magnifier->zoomfactorx, path, 	
-				    MAGNIFIER_ZOOM_FACTOR_X);
-    gnopiconf_set_int_in_section (magnifier->zoomfactory, path, 	
-				    MAGNIFIER_ZOOM_FACTOR_Y);
+    gnopiconf_set_double_in_section (magnifier->zoomfactorx, path, 	
+				     MAGNIFIER_ZOOM_FACTOR_X);
+    gnopiconf_set_double_in_section (magnifier->zoomfactory, path, 	
+				     MAGNIFIER_ZOOM_FACTOR_Y);
     gnopiconf_set_bool_in_section (magnifier->zoomfactor_lock, path, 
 				    MAGNIFIER_ZOOM_FACTOR_LOCK);
 
@@ -661,20 +670,20 @@ magconf_changes_cb			(GConfClient	*client,
 	else
 	if (strcmp (key, CONFIG_PATH MAGNIFIER_PATH MAGNIFIER_SCHEMA1 MAGNIFIER_ZOOM_FACTOR_X) == 0)
 	{
-	    if (entry->value->type == GCONF_VALUE_INT)
+	    if (entry->value->type == GCONF_VALUE_FLOAT)
 	    {
 		magnifier_setting->zoomfactorx = 
-		    gconf_value_get_int (entry->value);
+		    gconf_value_get_float (entry->value);
 		magui_set_zoomfactor_x(magnifier_setting);
 	    }
 	}
 	else
 	if (strcmp (key, CONFIG_PATH MAGNIFIER_PATH MAGNIFIER_SCHEMA1 MAGNIFIER_ZOOM_FACTOR_Y) == 0)
 	{
-	    if (entry->value->type == GCONF_VALUE_INT)
+	    if (entry->value->type == GCONF_VALUE_FLOAT)
 	    {
 		magnifier_setting->zoomfactory = 
-		    gconf_value_get_int (entry->value);
+		    gconf_value_get_float (entry->value);
 		magui_set_zoomfactor_y(magnifier_setting);
 	    }
 	}
