@@ -23,10 +23,14 @@
 #include <bonobo/bonobo-exception.h>
 #include <bonobo-activation/bonobo-activation.h>
 #include <libbonobo.h>
+#include <string.h>
 
 static CORBA_Environment ev;
 MagRectangle zp_rect;
 
+gchar *source_display = NULL;
+gchar *target_display = NULL;
+
 void
 check_return_value (CORBA_Environment 	*env, 
 		    int 		line)
@@ -68,6 +72,9 @@ get_magnifier (void)
 	g_warning ("Could not locate magnifier");
     }
     
+    source_display = NULL;
+    target_display = NULL;    
+    
     return (GNOME_Magnifier_Magnifier) oclient;
 }
 
@@ -83,6 +90,8 @@ magnifier_exit (GNOME_Magnifier_Magnifier	magnifier)
 					 &ev);
 */	GNOME_Magnifier_Magnifier_dispose (magnifier, &ev);
     }
+    g_free (source_display);
+    g_free (target_display);
 }
 
 /*________________________SOURCE/TARGET____________________________________*/
@@ -90,6 +99,8 @@ void
 magnifier_set_source_screen (GNOME_Magnifier_Magnifier 	magnifier,
 			     gchar 			*source_display_screen)
 {
+    g_free (source_display);
+    source_display = g_strdup (source_display_screen);
     if (magnifier)
     {
 	GNOME_Magnifier_Magnifier__set_SourceDisplay (magnifier,
@@ -103,6 +114,8 @@ void
 magnifier_set_target_screen (GNOME_Magnifier_Magnifier 	magnifier,
 			     gchar 			*target_display_screen)
 {
+    g_free (target_display);
+    target_display = g_strdup (target_display_screen);
     if (magnifier)
     {
 	    GNOME_Magnifier_Magnifier__set_TargetDisplay (magnifier,
@@ -130,7 +143,7 @@ magnifier_get_source (GNOME_Magnifier_Magnifier 	magnifier,
 	if (properties)
 	{
 	    rectangle_any = Bonobo_PropertyBag_getValue (properties,
-    			    				 "source-display-size",
+    			    				 "source-display-bounds",
   							 &ev);
 	    check_return_value (&ev, __LINE__);
 	    rectangle = (GNOME_Magnifier_RectBounds *) rectangle_any->_value;
@@ -403,34 +416,42 @@ mag_rect_clip_to_source (MagRectangle *roi)
 {    
     static MagRectangle temp;
     
-    if ((roi->width > zp_rect.left) && (roi->width < zp_rect.width))
+    g_assert (source_display);
+    g_assert (target_display);
+    
+    if (strcmp (source_display, target_display) == 0)
     {
-	if ((roi->top < zp_rect.height) && (roi->height > zp_rect.top))
+	if ((roi->width > zp_rect.left) && (roi->width < zp_rect.width))
 	{
-	    temp.width = zp_rect.left;
-	    temp.top = roi->top;
-	    temp.left = roi->left - (roi->width - zp_rect.left);
-	    temp.height = roi->height;
-	}
+	    if ((roi->top < zp_rect.height) && (roi->height > zp_rect.top))
+	    {
+		temp.width = zp_rect.left;
+		temp.top = roi->top;
+		temp.left = roi->left - (roi->width - zp_rect.left);
+		temp.height = roi->height;
+	    }
 	else
 	    return roi;    
+	}
+	else
+	    if ((roi->left < zp_rect.width) && (roi->left > zp_rect.left))
+	    {
+		if ((roi->top < zp_rect.height) && (roi->height > zp_rect.top))
+		{
+		    temp.width = (zp_rect.width - roi->left) + roi->width;
+		    temp.top = roi->top;
+		    temp.left = zp_rect.width;
+		    temp.height = roi->height;
+		}
+    		else
+		    return roi;    
+	    }
+	    else 
+		return roi;
+	return &temp;
     }
     else
-    if ((roi->left < zp_rect.width) && (roi->left > zp_rect.left))
-    {
-	if ((roi->top < zp_rect.height) && (roi->height > zp_rect.top))
-	{
-	    temp.width = (zp_rect.width - roi->left) + roi->width;
-	    temp.top = roi->top;
-	    temp.left = zp_rect.width;
-	    temp.height = roi->height;
-	}
-	else
-	    return roi;    
-    }
-    else 
 	return roi;
-    return &temp;
 }
 
 
