@@ -65,7 +65,8 @@ static SRCBraille *src_braille = NULL;
 
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
-extern void brl_input_event (BrlInEvent *event);
+extern gchar* scroll;
+extern void brl_input_event (BRLInEvent *event);
 extern gchar *fmt_for_xml (gchar *, gchar *, gchar *);
 
 gint 	brlmon_sd, 	/* BrlMonitor socket descriptor*/
@@ -106,7 +107,7 @@ udp_sock_glib_cb(GIOChannel *source,
 		GIOCondition condition, 
 		gpointer data)
 {	
-    BrlInEvent event;
+    BRLInEvent event;
     BrlPackage package;
     gint clilen;
     gint n;
@@ -544,11 +545,23 @@ src_brlmon_show (gchar *message)
 		/* main display */
 		"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">\n",
 		text,
-		"</BRLDISP>\n",
-		/* nothing to do on state display here*/
-		"</BRLOUT>\n", 
 		NULL);
-
+    if (scroll && scroll[0])
+    {
+	brlout = g_strconcat (brlout,
+	                      "<SCROLL mode=\"", scroll, "\" /SCROLL>\n",
+		              "</BRLDISP>\n",
+		              "</BRLOUT>\n",
+		              NULL);
+    }
+    else
+    {		
+	brlout = g_strconcat (brlout,
+		              "</BRLDISP>\n",
+		              "</BRLOUT>\n", 
+    		              NULL);
+    }		    
+    		
     src_brlmon_send  (brlout);
     g_free (brlout);						
     g_free(text);
@@ -675,7 +688,7 @@ brl_mon_run_process (void)
     }
     else
     {
-	gchar *args[] = {"brlmonitor", NULL};
+	gchar *args[] = {BRLMON_DIR"brlmonitor", NULL};
 	if (!g_spawn_async ( ".", args , NULL , 
 			    G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
 			    NULL, NULL, &src_brlmon_pid, NULL))
