@@ -36,19 +36,25 @@
 #include "srpres.h"
 #include "screen-review.h"
 #include "srintl.h"
+#include <gdk/gdkx.h>
 
 #undef SRCTRL_DEBUG
 
+#define ZOOM_FACTOR_INC (0.5)
+
 extern gboolean src_use_magnifier;
 extern gboolean src_use_speech;
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
 extern gboolean src_enable_format_string;
 
+gchar    *scroll = NULL;
+
 gboolean  src_button_left_pressed   = FALSE;
 gboolean  src_button_middle_pressed = FALSE;
 gboolean  src_button_right_pressed  = FALSE;
 
+static gboolean src_xevie_present = FALSE;
 
 /*___________________________< MAGNIFIER CONTROL>_____________________________*/
 static gboolean
@@ -56,53 +62,60 @@ src_mag_lock_xy_scale ()
 {
     if (src_use_magnifier) 
     {
-	gboolean lock = TRUE;
-	gboolean default_lock = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
-					   CFGT_BOOL, 
-					   &lock, 
-					   (gpointer)&default_lock,
-					   MAGNIFIER_CONFIG_PATH)
+	if (login_time)
+	{
+	    src_magnifier_set_zoom_factor_lock_on_off();
+	}
+	else
+	{
+	    gboolean lock = TRUE;
+	    gboolean default_lock = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
+					       CFGT_BOOL, 
+					       &lock, 
+					       (gpointer)&default_lock,
+					       MAGNIFIER_CONFIG_PATH)
 	    ) return FALSE;
 	
-	lock  = !lock;
-	if (lock)
-	{
-	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	    gint zoom_factory;
-	    gint zoom_factorx;
-	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-					   CFGT_INT, 
-					   &zoom_factory, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-					   CFGT_INT, 
-					   &zoom_factorx, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    lock  = !lock;
+	    if (lock)
+	    {
+		gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+		gint zoom_factory;
+		gint zoom_factorx;
+		if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
+						   CFGT_INT, 
+						   &zoom_factory, 
+						   (gpointer)&default_val,
+						   MAGNIFIER_CONFIG_PATH)
+		    ) return FALSE;
+		if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
+						   CFGT_INT, 
+						   &zoom_factorx, 
+						   (gpointer)&default_val,
+						   MAGNIFIER_CONFIG_PATH)
+		    ) return FALSE;
 		
-	    zoom_factorx = 
-		zoom_factory = (zoom_factory + zoom_factorx) >> 1;
+		zoom_factorx = 
+		    zoom_factory = (zoom_factory + zoom_factorx) >> 1;
 
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		    ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		    ) return FALSE;
+	    }
+	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_LOCK,
+	    		          CFGT_BOOL, 
+			          &lock,
+			          MAGNIFIER_CONFIG_PATH)
 		) return FALSE;
 	}
-	if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_LOCK,
-			      CFGT_BOOL, 
-			      &lock,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
     }
     return TRUE;
 }
@@ -112,50 +125,56 @@ src_mag_increase_y_scale ()
 {
    if (src_use_magnifier)
     {
-	gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	gint zoom_factory;
-	gboolean zoom_factor_lock;
-	gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
-
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-					   CFGT_INT, 
-					   &zoom_factory, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
-					   CFGT_BOOL, 
-					   &zoom_factor_lock, 
-					   (gpointer)&zoom_factor_lock_def,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-	zoom_factory = ((++zoom_factory) > MAX_ZOOM_FACTOR_Y) ? MAX_ZOOM_FACTOR_Y : zoom_factory;
-	
-	if (!zoom_factor_lock)
+	if (login_time)
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    src_magnifier_increase_y_scale ();
 	}
 	else
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gint zoom_factory;
+	    gboolean zoom_factor_lock;
+	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
+
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
+					       CFGT_INT, 
+					       &zoom_factory, 
+					       (gpointer)&default_val,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
+					       CFGT_BOOL, 
+					       &zoom_factor_lock, 
+					       (gpointer)&zoom_factor_lock_def,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+
+	    zoom_factory = ((++zoom_factory) > MAX_ZOOM_FACTOR_Y) ? MAX_ZOOM_FACTOR_Y : zoom_factory;
+	
+	    if (!zoom_factor_lock)
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
+	    else
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
 	}
     }
     return TRUE;
-
 }
 
 
@@ -164,100 +183,113 @@ src_mag_decrease_y_scale ()
 {
    if (src_use_magnifier)
     {
-	gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	gint zoom_factory;
-	gboolean zoom_factor_lock;
-	gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
-
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-					   CFGT_INT, 
-					   &zoom_factory, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
-					   CFGT_BOOL, 
-					   &zoom_factor_lock, 
-					   (gpointer)&zoom_factor_lock_def,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-	zoom_factory = ((--zoom_factory) < MIN_ZOOM_FACTOR_Y) ? MIN_ZOOM_FACTOR_Y : zoom_factory;
-
-	if (!zoom_factor_lock)
+	if (login_time)
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    src_magnifier_decrease_y_scale ();
 	}
 	else
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factory,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gint zoom_factory;
+	    gboolean zoom_factor_lock;
+	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
+
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
+					       CFGT_INT, 
+					       &zoom_factory, 
+					       (gpointer)&default_val,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
+					       CFGT_BOOL, 
+					       &zoom_factor_lock, 
+					       (gpointer)&zoom_factor_lock_def,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+
+	    zoom_factory = ((--zoom_factory) < MIN_ZOOM_FACTOR_Y) ? MIN_ZOOM_FACTOR_Y : zoom_factory;
+
+	    if (!zoom_factor_lock)
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
+	    else
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factory,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
 	}
     }
     return TRUE;
-
 }
 
 
 static gboolean
 src_mag_increase_x_scale ()
 {
-     if (src_use_magnifier)
+    
+    if (src_use_magnifier)
     {
-	gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	gint zoom_factorx;
-	gboolean zoom_factor_lock;
-	gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
-
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-					   CFGT_INT, 
-					   &zoom_factorx, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
-					   CFGT_BOOL, 
-					   &zoom_factor_lock, 
-					   (gpointer)&zoom_factor_lock_def,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-	zoom_factorx = ((++zoom_factorx) > MAX_ZOOM_FACTOR_X) ? MAX_ZOOM_FACTOR_X : zoom_factorx;
-
-	if (!zoom_factor_lock)
+	if (login_time)
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    src_magnifier_increase_x_scale ();
 	}
 	else
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	}
+	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gint zoom_factorx;
+	    gboolean zoom_factor_lock;
+	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
 
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
+					       CFGT_INT, 
+					       &zoom_factorx, 
+					       (gpointer)&default_val,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
+					       CFGT_BOOL, 
+					       &zoom_factor_lock, 
+					       (gpointer)&zoom_factor_lock_def,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+
+	    zoom_factorx = ((++zoom_factorx) > MAX_ZOOM_FACTOR_X) ? MAX_ZOOM_FACTOR_X : zoom_factorx;
+
+	    if (!zoom_factor_lock)
+	    {
+	        if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
+    	    else
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+	    	   ) return FALSE;
+	    }
+	}
     }
     return TRUE;
 }
@@ -266,48 +298,55 @@ src_mag_increase_x_scale ()
 static gboolean
 src_mag_decrease_x_scale ()
 {
-   if (src_use_magnifier)
+    if (src_use_magnifier)
     {
-	gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	gint zoom_factorx;
-	gboolean zoom_factor_lock;
-	gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
-
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-					   CFGT_INT, 
-					   &zoom_factorx, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
-					   CFGT_BOOL, 
-					   &zoom_factor_lock, 
-					   (gpointer)&zoom_factor_lock_def,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-	zoom_factorx = ((--zoom_factorx) < MIN_ZOOM_FACTOR_X) ? MIN_ZOOM_FACTOR_X : zoom_factorx;
-
-	if (!zoom_factor_lock)
+	if (login_time)
 	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+	    src_magnifier_decrease_x_scale ();
 	}
-	else
-	{
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
-	    if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				  CFGT_INT, 
-				  &zoom_factorx,
-				  MAGNIFIER_CONFIG_PATH)
-		) return FALSE;
+        else
+	{	
+            gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gint zoom_factorx;
+	    gboolean zoom_factor_lock;
+	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
+
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
+					       CFGT_INT, 
+					       &zoom_factorx, 
+					       (gpointer)&default_val,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_LOCK, 
+					       CFGT_BOOL, 
+					       &zoom_factor_lock, 
+					       (gpointer)&zoom_factor_lock_def,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+
+	    zoom_factorx = ((--zoom_factorx) < MIN_ZOOM_FACTOR_X) ? MIN_ZOOM_FACTOR_X : zoom_factorx;
+
+	    if (!zoom_factor_lock)
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
+	    else
+	    {
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+				      CFGT_INT, 
+				      &zoom_factorx,
+				      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
 	}
     }
     return TRUE;
@@ -328,13 +367,18 @@ src_mag_mouse_tracking_toggle ()
     index = (index + 1) % G_N_ELEMENTS (mouse_tracking);
     if (src_use_magnifier) 
     {
-
-	if (!srconf_set_data (MAGNIFIER_MOUSE_TRACKING,
-			      CFGT_STRING, 
-			      mouse_tracking[index],
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	
+	if (login_time)
+	{
+	    src_magnifier_set_mouse_tracking_mode (mouse_tracking[index]);
+	}
+	else
+	{
+    	    if (!srconf_set_data (MAGNIFIER_MOUSE_TRACKING,
+			          CFGT_STRING, 
+			          mouse_tracking[index],
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	}
     }
     return TRUE;
 }
@@ -365,22 +409,30 @@ src_mag_cursor_toggle ()
 	};
     gboolean cursor;
     gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+    
+    index = (index + 1) % G_N_ELEMENTS (cursors);
+    if (login_time)
+    {
+	src_magnifier_set_cursor_name (cursors[index]);
+    }
+    else
+    {
+	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
 					   CFGT_BOOL, 
 					   &cursor, 
 					   (gpointer)&default_cursor,
 					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+	   ) return FALSE;
 
-    if (src_use_magnifier && cursor) 
-    {
-	index = (index + 1) % G_N_ELEMENTS (cursors);
-	if (!srconf_set_data (MAGNIFIER_CURSOR_NAME,
-			      CFGT_STRING, 
-			      cursors[index],
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+	if (src_use_magnifier && cursor) 
+	{
+	   if (!srconf_set_data (MAGNIFIER_CURSOR_NAME,
+			         CFGT_STRING, 
+			         cursors[index],
+			         MAGNIFIER_CONFIG_PATH)
+	      ) return FALSE;
 	
+	}
     }
     return TRUE;
 }
@@ -388,36 +440,38 @@ src_mag_cursor_toggle ()
 static gboolean
 src_mag_cursor_adjust_size (gint offset)
 {
-    gboolean cursor;
-    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-				       CFGT_BOOL, 
-				       &cursor, 
-				       (gpointer)&default_cursor,
-				       MAGNIFIER_CONFIG_PATH)
-	) return FALSE;
-
-    if (src_use_magnifier && cursor) 
+    if (src_use_magnifier)
     {
-	gint default_val = DEFAULT_MAGNIFIER_CURSOR_SIZE;
-	gint cursor_size;
+	gboolean cursor;
+	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
+	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+			    		   CFGT_BOOL, 
+				           &cursor, 
+				           (gpointer)&default_cursor,
+				           MAGNIFIER_CONFIG_PATH)
+           ) return FALSE;
+
+        if (cursor) 
+        {
+    	    gint default_val = DEFAULT_MAGNIFIER_CURSOR_SIZE;
+	    gint cursor_size;
 	
 	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SIZE, 
-					   CFGT_INT, 
-					   &cursor_size, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
+				           CFGT_INT, 
+				           &cursor_size, 
+				           (gpointer)&default_val,
+				    	   MAGNIFIER_CONFIG_PATH)
+	   ) return FALSE;
+		
 	cursor_size = CLAMP (cursor_size + offset,
 			     MIN_CROSSWIRE_SIZE,
 			     MAX_CROSSWIRE_SIZE);
-
 	if (!srconf_set_data (MAGNIFIER_CURSOR_SIZE, 
-			      CFGT_INT, 
-			      &cursor_size, 
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+		    	      CFGT_INT, 
+		    	      &cursor_size, 
+		    	      MAGNIFIER_CONFIG_PATH)
+	   ) return FALSE;
+	}	   	
     }
     return TRUE;
 }
@@ -425,45 +479,62 @@ src_mag_cursor_adjust_size (gint offset)
 static gboolean
 src_mag_cursor_decrease_size ()
 {
-    return src_mag_cursor_adjust_size (-4);
+    if (login_time)
+        return src_magnifier_adjust_cursor_size (-4);
+    else
+    	return src_mag_cursor_adjust_size (-4);
 }
 
 static gboolean
 src_mag_cursor_increase_size ()
 {
-    return src_mag_cursor_adjust_size (4);
+    if (login_time)
+    	return src_magnifier_adjust_cursor_size (+4);
+    else
+    	return src_mag_cursor_adjust_size (+4);
 }
 
 static gboolean
 src_mag_cursor_mag_on_off ()
 {
-	gboolean cursor;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+    gboolean cursor;
+    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
 
-     if (src_use_magnifier && cursor)
+    if (src_use_magnifier)
     {
-	gboolean cursor_mag;
-	gboolean default_cursor_mag = DEFAULT_MAGNIFIER_CURSOR_SCALE;
+	if (login_time)
+	{
+	    src_magnifier_set_cursor_magnification_on_off ();
+	}
+	else
+	{
+	    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+					       CFGT_BOOL, 
+					       &cursor, 
+					       (gpointer)&default_cursor,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
 
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SCALE, 
-					   CFGT_BOOL, 
-					   &cursor_mag, 
-					   (gpointer)&default_cursor_mag,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	cursor_mag = !cursor_mag;
+            if (cursor)
+	    {
+		gboolean cursor_mag;
+		gboolean default_cursor_mag = DEFAULT_MAGNIFIER_CURSOR_SCALE;
+
+		if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SCALE, 
+					           CFGT_BOOL, 
+					           &cursor_mag, 
+					           (gpointer)&default_cursor_mag,
+						   MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	        cursor_mag = !cursor_mag;
 	
-	if (!srconf_set_data (MAGNIFIER_CURSOR_SCALE, 
-			      CFGT_BOOL, 
-			      &cursor_mag, 
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+		if (!srconf_set_data (MAGNIFIER_CURSOR_SCALE, 
+			    	      CFGT_BOOL, 
+			              &cursor_mag, 
+			    	      MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	    }
+	 }
     }
     return TRUE; 
 }
@@ -515,46 +586,62 @@ src_mag_crosswire_adjust_size (guint offset)
 static gboolean
 src_mag_crosswire_decrease_size ()
 {
-	return src_mag_crosswire_adjust_size (-1);
+    if (login_time)
+    	return src_magnifier_adjust_crosswire_size (-1);
+    else
+    	return src_mag_crosswire_adjust_size (-1);
+    
 }
 
 static gboolean
 src_mag_crosswire_increase_size ()
 {
-	return src_mag_crosswire_adjust_size (+1);
+    if (login_time)
+    	return src_magnifier_adjust_crosswire_size (+1);
+    else
+        return src_mag_crosswire_adjust_size (+1);
 }
 
 static gboolean
 src_mag_crosswire_on_off ()
 {
-	gboolean cursor;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-    if (src_use_magnifier && cursor)
+    gboolean cursor;
+    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
+    
+    if (src_use_magnifier)
     {
-	gboolean crosswire;
-	gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE,
-					   CFGT_BOOL,
-					   &crosswire,
-					   (gpointer)&default_crosswire ,
-					   MAGNIFIER_CONFIG_PATH)
-	    )
-	    return FALSE;
-	crosswire = !crosswire;
+	if (login_time)
+	{
+	    src_magnifier_set_crosswire_on_off ();
+	}
+	else
+	{
+	    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+					       CFGT_BOOL, 
+					       &cursor, 
+					       (gpointer)&default_cursor,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (cursor)
+	    {
+		gboolean crosswire;
+		gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
+		if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE,
+						   CFGT_BOOL,
+						   &crosswire,
+						   (gpointer)&default_crosswire ,
+						   MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+	
+	    crosswire = !crosswire;
 
-	if (!srconf_set_data (MAGNIFIER_CROSSWIRE,
-			      CFGT_BOOL,
-			      &crosswire,
-			      MAGNIFIER_CONFIG_PATH)
-	    )
-	    
-	return FALSE;
+	    if (!srconf_set_data (MAGNIFIER_CROSSWIRE,
+			    	  CFGT_BOOL,
+			          &crosswire,
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    }
+	}
     }
     return TRUE;
 }
@@ -562,41 +649,51 @@ src_mag_crosswire_on_off ()
 static gboolean
 src_mag_crosswire_clip_on_off ()
 {
-	gboolean cursor, crosswire;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
-					   CFGT_BOOL, 
-					   &crosswire, 
-					   (gpointer)&default_crosswire,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-    if (src_use_magnifier && cursor && crosswire)
+    gboolean cursor, crosswire;
+    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
+    gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
+    
+    if (src_use_magnifier)
     {
-	gboolean crosswire_clip;
-	gboolean default_crosswire_clip = DEFAULT_MAGNIFIER_CROSSWIRE_CLIP;
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_CLIP,
-					   CFGT_BOOL,
-					   &crosswire_clip,
-					   (gpointer)&default_crosswire_clip ,
-					   MAGNIFIER_CONFIG_PATH)
-	    )
-	    return FALSE;
-	crosswire_clip = !crosswire_clip;
+	if (login_time)
+	{
+	    src_magnifier_set_crosswire_clip_on_off ();
+	}
+	else
+	{
+	    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+					       CFGT_BOOL, 
+					       &cursor, 
+					       (gpointer)&default_cursor,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
+					       CFGT_BOOL, 
+					       &crosswire, 
+					       (gpointer)&default_crosswire,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+
+	    if (cursor && crosswire)
+	    {
+		gboolean crosswire_clip;
+		gboolean default_crosswire_clip = DEFAULT_MAGNIFIER_CROSSWIRE_CLIP;
+		if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_CLIP,
+						   CFGT_BOOL,
+						   &crosswire_clip,
+						   (gpointer)&default_crosswire_clip ,
+						   MAGNIFIER_CONFIG_PATH)
+		   ) return FALSE;
+		crosswire_clip = !crosswire_clip;
 	
-	if (!srconf_set_data (MAGNIFIER_CROSSWIRE_CLIP,
-			      CFGT_BOOL,
-			      &crosswire_clip,
-			      MAGNIFIER_CONFIG_PATH)
-	    )
-	    return FALSE;
+		if (!srconf_set_data (MAGNIFIER_CROSSWIRE_CLIP,
+			    	      CFGT_BOOL,
+			    	      &crosswire_clip,
+				      MAGNIFIER_CONFIG_PATH)
+		   )
+		return FALSE;
+	    }
+	}
     }
     return TRUE;
 }
@@ -609,34 +706,28 @@ src_mag_cursor_on_off ()
     {
 	gboolean cursor;
 	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-/*	gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;*/
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-/*	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
-					   CFGT_BOOL, 
-					   &crosswire, 
-					   (gpointer)&default_crosswire,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-*/
-	cursor = !cursor;
 	
-	if (!srconf_set_data (MAGNIFIER_CURSOR,
-			      CFGT_BOOL, 
-			      &cursor,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-/*	if (!srconf_set_data (MAGNIFIER_CROSSWIRE,
-			      CFGT_BOOL, 
-			      &crosswire,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-*/
+	if (login_time)
+	{
+	    src_magnifier_set_cursor_state_on_off ();
+	}
+	else
+	{
+	    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+					       CFGT_BOOL, 
+					       &cursor, 
+					       (gpointer)&default_cursor,
+					       MAGNIFIER_CONFIG_PATH)
+		) return FALSE;
+
+	    cursor = !cursor;
 	
+	    if (!srconf_set_data (MAGNIFIER_CURSOR,
+			          CFGT_BOOL, 
+			          &cursor,
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	}
     }
     return TRUE;
 }
@@ -648,24 +739,29 @@ src_mag_panning_on_off ()
 {
     if (src_use_magnifier) 
     {
-	gboolean panning;
-	gboolean default_panning = DEFAULT_MAGNIFIER_PANNING;
-	if (!srconf_get_data_with_default (MAGNIFIER_PANNING, 
-					   CFGT_BOOL, 
-					   &panning, 
-					   (gpointer)&default_panning, 
-					   MAGNIFIER_CONFIG_PATH)
-	    ) 
-	    return FALSE;
+	if (login_time)
+	{
+	    src_magnifier_set_panning_state ();
+	}
+	else
+	{
+	    gboolean panning;
+	    gboolean default_panning = DEFAULT_MAGNIFIER_PANNING;
+	    if (!srconf_get_data_with_default (MAGNIFIER_PANNING, 
+					       CFGT_BOOL, 
+					       &panning, 
+					       (gpointer)&default_panning, 
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
 
-	panning = !panning;
+	    panning = !panning;
 
-	if (!srconf_set_data (MAGNIFIER_PANNING,
-			      CFGT_BOOL, 
-			      &panning, 
-			      MAGNIFIER_CONFIG_PATH)
-	    ) 
-	    return FALSE;
+	    if (!srconf_set_data (MAGNIFIER_PANNING,
+			          CFGT_BOOL, 
+			          &panning, 
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	}
     }
     return TRUE;
 }
@@ -676,22 +772,29 @@ src_mag_invert_on_off ()
 {
     if (src_use_magnifier) 
     {
-	gboolean invert = TRUE;
-	gboolean default_invert = DEFAULT_MAGNIFIER_INVERT;
-	if (!srconf_get_data_with_default (MAGNIFIER_INVERT, 
-					   CFGT_BOOL, 
-					   &invert, 
-					   (gpointer)&default_invert,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+	if (login_time)
+	{
+	    src_magnifier_set_invert_state ();
+	}
+	else
+	{
+	    gboolean invert = TRUE;
+	    gboolean default_invert = DEFAULT_MAGNIFIER_INVERT;
+	    if (!srconf_get_data_with_default (MAGNIFIER_INVERT, 
+					       CFGT_BOOL, 
+					       &invert, 
+					       (gpointer)&default_invert,
+					       MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
 
-	invert  = !invert;
+    	    invert  = !invert;
 	
-	if (!srconf_set_data (MAGNIFIER_INVERT,
-			      CFGT_BOOL, 
-			      &invert,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+	    if (!srconf_set_data (MAGNIFIER_INVERT,
+			          CFGT_BOOL, 
+			          &invert,
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	}
     }
     return TRUE;
 }
@@ -720,11 +823,18 @@ src_mag_smoothing_toggle ()
     
     if (src_use_magnifier) 
     {
-	if (!srconf_set_data (MAGNIFIER_SMOOTHING,
-			      CFGT_STRING, 
-			      smoothing[index],
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+	if (login_time)
+	{
+	    src_magnifier_set_smoothing (smoothing[index]);
+	}
+	else
+	{
+	    if (!srconf_set_data (MAGNIFIER_SMOOTHING,
+			          CFGT_STRING, 
+			          smoothing[index],
+			          MAGNIFIER_CONFIG_PATH)
+	       ) return FALSE;
+	}
     }
     return TRUE;    
 }
@@ -1151,7 +1261,11 @@ src_find_set ()
 			  CFGT_STRING, 
 			  "find_set",
 			  SRCORE_PATH)
-        ) return FALSE;
+        )
+    {
+	if (!login_time)
+	     return FALSE;
+    }
     return TRUE;
 }
 
@@ -1327,6 +1441,8 @@ src_nav_change_mode ()
 
 
 static gchar *src_last_cmd = NULL;
+static gchar *src_before_last_cmd = NULL;
+
 extern gint src_speech_mode;
 static gboolean src_repeat_cnt = 1;
 
@@ -1342,7 +1458,11 @@ src_repeat_last ()
     gint mode, last_mode;
     
     if (!src_use_speech)
+    {    
+	src_cmd_queue_add (src_before_last_cmd);
+	src_cmd_queue_process ();
 	return FALSE;
+    }	
 
     if (src_repeat_cnt == 1)
 	mode = SRC_SPEECH_SPELL_NORMAL;
@@ -1516,13 +1636,21 @@ src_nav_statusbar ()
 
 extern gboolean SPI_generateMouseEvent (long int x, long int y, gchar *name);
 
+static gboolean
+src_generateMouseEvent (long int x, long int y, gchar *name)
+{
+    if (src_xevie_present)
+	return SPI_generateMouseEvent (x, y, name);
+    gdk_beep ();
+    return FALSE;
+}
 
 static gboolean
 src_mouse_left_press ()
 {
     if (src_button_left_pressed) return FALSE;
     src_button_left_pressed = !src_button_left_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b1p");
+    return src_generateMouseEvent (-1, -1, "b1p");
 }
 
 static gboolean
@@ -1530,21 +1658,21 @@ src_mouse_left_release ()
 {
     if (!src_button_left_pressed) return FALSE;
     src_button_left_pressed = !src_button_left_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b1r");
+    return src_generateMouseEvent (-1, -1, "b1r");
 }
 
 static gboolean
 src_mouse_left_click ()
 {
     if (src_button_left_pressed) return FALSE;
-    return SPI_generateMouseEvent (-1, -1, "b1c");
+    return src_generateMouseEvent (-1, -1, "b1c");
 }
 
 /*
 static gboolean
 src_mouse_left_double_click ()
 {
-    return SPI_generateMouseEvent (-1, -1, "b1d");
+    return src_generateMouseEvent (-1, -1, "b1d");
 }
 */
 
@@ -1553,7 +1681,7 @@ src_mouse_middle_press ()
 {
     if (src_button_middle_pressed) return FALSE;
     src_button_middle_pressed = !src_button_middle_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b3p");
+    return src_generateMouseEvent (-1, -1, "b3p");
 }
 
 static gboolean
@@ -1561,21 +1689,21 @@ src_mouse_middle_release ()
 {
     if (!src_button_middle_pressed) return FALSE;
     src_button_middle_pressed = !src_button_middle_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b3r");
+    return src_generateMouseEvent (-1, -1, "b3r");
 }
 
 static gboolean
 src_mouse_middle_click ()
 {
     if (src_button_middle_pressed) return FALSE;
-    return SPI_generateMouseEvent (-1, -1, "b3c");
+    return src_generateMouseEvent (-1, -1, "b3c");
 }
 
 /*
 static gboolean
 src_mouse_middle_double_click ()
 {
-    return SPI_generateMouseEvent (-1, -1, "b3d");
+    return src_generateMouseEvent (-1, -1, "b3d");
 }
 */
 
@@ -1585,7 +1713,7 @@ src_mouse_right_press ()
 {
     if (src_button_right_pressed) return FALSE;
     src_button_right_pressed = !src_button_right_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b2p");
+    return src_generateMouseEvent (-1, -1, "b2p");
 }
 
 static gboolean
@@ -1593,21 +1721,21 @@ src_mouse_right_release ()
 {
     if (!src_button_right_pressed) return FALSE;
     src_button_right_pressed = !src_button_right_pressed;
-    return SPI_generateMouseEvent (-1, -1, "b2r");
+    return src_generateMouseEvent (-1, -1, "b2r");
 }
 
 static gboolean
 src_mouse_right_click ()
 {
     if (src_button_right_pressed) return FALSE;
-    return SPI_generateMouseEvent (-1, -1, "b2c");
+    return src_generateMouseEvent (-1, -1, "b2c");
 }
 
 /*
 static gboolean
 src_mouse_right_double_click ()
 {
-    return SPI_generateMouseEvent (-1, -1, "b2d");
+    return src_generateMouseEvent (-1, -1, "b2d");
 }
 */
 
@@ -2438,9 +2566,23 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 	sprintf (offset, "%d", braille_offset);
 	brlout = g_strconcat ("<BRLOUT language=\"", braille_translation_table, "\" bstyle=\"8\" clear=\"true\">",
 				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">",
-				brlout, "</BRLDISP>",
-				"</BRLOUT>",
+				brlout, 
 				NULL);
+	if (scroll)
+	{
+	    brlout = g_strconcat (brlout, 
+	    	            	  "<SCROLL mode=\"", scroll, "\"></SCROLL>",
+				  "</BRLDISP>",
+				  "</BRLOUT>",
+				  NULL);
+	}  
+	else
+	{
+	    brlout = g_strconcat (brlout, 
+				  "</BRLDISP>",
+				  "</BRLOUT>",
+				  NULL);
+	}			
 	g_free (tmp);
 	if (src_use_braille)
 	    src_braille_send (brlout);
@@ -2956,15 +3098,13 @@ src_pause_resume ()
     return TRUE;
 }
 
-
 static gboolean
 src_braille_scroll_char_left ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MAX (0, offset - 1));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("-1");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -2976,10 +3116,9 @@ static gboolean
 src_braille_scroll_char_right ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MIN (1024, offset + 1));
+      if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("1");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -2993,10 +3132,9 @@ static gboolean
 src_braille_scroll_display_left ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MAX (0, offset - 20));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("-width");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3010,10 +3148,9 @@ static gboolean
 src_braille_scroll_display_right ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MIN (1024, offset + 20));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("width");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3032,7 +3169,7 @@ src_braille_on_off ()
 			  &braille_on_off,
 			  SRCORE_PATH)
         ) return FALSE;
-	
+    	
     return TRUE;
 }
 
@@ -3263,7 +3400,7 @@ src_cmd_process (gchar *cmd)
 
 gboolean
 src_cmd_queue_process ()
-{
+{	
     sru_assert (src_cmd_queue);
 
     while (!g_queue_is_empty (src_cmd_queue))
@@ -3271,10 +3408,14 @@ src_cmd_queue_process ()
 	gchar *cmd;
 	/* preprocess script stuff */
 	src_cmd_queue_remove (&cmd);
+	
 	if (cmd)
 	{
 	    static gboolean repeat = FALSE;
 	    
+	    if (src_last_cmd && strcmp (src_last_cmd, "present layer changed") != 0)
+		src_before_last_cmd = src_last_cmd;
+		
 	    if (strcmp (cmd, "widget surroundings") 	== 0 ||
 		strcmp (cmd, "present current object") 	== 0 ||
 		strcmp (cmd, "present current window") 	== 0 ||
@@ -3309,6 +3450,12 @@ src_cmd_queue_process ()
 	    if (strcmp (cmd, "repeat last") == 0)
 	    	repeat = TRUE;
 	
+	    
+#ifdef SRCTRL_DEBUG	    
+	    fprintf (stderr, "\n   cmd: %s", cmd);	
+	    fprintf (stderr, "\n   last_cmd: %s", src_last_cmd);	
+	    fprintf (stderr, "\n   before_last_cmd: %s", src_before_last_cmd);	
+#endif	    
 	    src_cmd_process (cmd);
 	    repeat = FALSE;
 	}
@@ -3519,6 +3666,7 @@ src_ctrl_key_cmds_reload ()
 gboolean
 src_ctrl_init ()
 {
+    int mop, fer, fev;
     src_cmd_function_hash_init ();
     src_key_cmds_hash_init ();
     
@@ -3526,8 +3674,10 @@ src_ctrl_init ()
     src_key_queue = g_queue_new ();
 
     src_last_cmd = NULL;
+    src_before_last_cmd = NULL;
     src_repeat_cnt = 1;
-    
+    src_xevie_present = XQueryExtension (GDK_DISPLAY (), "XEVIE", &mop, &fev, &fer);
+
     return TRUE;
 }
 
@@ -3548,6 +3698,7 @@ src_ctrl_terminate ()
     src_key_cmds_hash = NULL;
     
     src_last_cmd = NULL;
+    src_before_last_cmd = NULL;
     src_repeat_cnt = 1;
     
     return TRUE;
