@@ -29,6 +29,7 @@
 #include "srintl.h"
 #include <libxml/parser.h>
 
+extern gint src_caret_position;
 
 typedef enum
 {
@@ -1189,7 +1190,9 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 			if (info)
 			{
 			    add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, info);
-			    g_free (info);
+			    if (chunk[i]->info != SRC_PRESENTATION_TEXT_LINE)
+					src_caret_position += strlen (info);
+				g_free (info);
 			}
 		    }
 		}
@@ -1246,7 +1249,10 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 		    {
 			add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, (gchar *)chunk[i]->data1);
 			if (info)
+			{
+				src_caret_position += strlen (info);
 			    g_free (info);
+			}
 		    }
 		    else
 		    {
@@ -1267,6 +1273,7 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 		    if (info2)
 		    {
 			add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, info2);
+			src_caret_position +=  strlen (info2);
 			g_free (info2);
 		    }
 		}
@@ -1281,7 +1288,10 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 			else
 			    add = src_xml_format ("TEXT", has_voice ? src_speech_get_voice ((gchar *)chunk[i]->voice) : NULL, (gchar *)chunk[i]->data2);
 			if (info)
-			    g_free (info);
+			{
+			   src_caret_position += strlen (info);
+			   g_free (info);
+			}
 		    }
 		    else
 		    {
@@ -1302,6 +1312,7 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 			if (tmp)
 			{
 			    add = src_xml_make_part ("DOTS", NULL, tmp);
+				src_caret_position += strlen (tmp);
 			    g_free (tmp);
 			}
 		    }
@@ -1315,6 +1326,7 @@ src_presentation_sro_get_pres_for_device_ (SRObject *obj,
 	    gchar *tmp = rv;
 	    if (strcmp (device, "braille") == 0)
 	    {
+		src_caret_position ++;
 		rv = g_strconcat (rv, rv[0] != '\0' ? "<TEXT> </TEXT>" : "", add, NULL); 
 	    }
 	    else
