@@ -417,7 +417,9 @@ gboolean sro_get_index_in_parent(const SRObject *obj, guint32 *index);
 gboolean sro_get_children_count	(const SRObject *obj, guint32 *children_count);
 gboolean sro_get_i_child	(const SRObject *obj, SRLong index, 
 					    SRObject **child);
-
+					    
+gboolean sro_get_app_name       (SRObject *obj, gchar **name, SRLong index); 
+gboolean sro_get_window_name    (SRObject *obj, gchar **role, gchar **name, SRLong index);
 gboolean sro_get_location	(SRObject *obj, SRCoordinateType type, 
 					    SRRectangle *location, SRLong index);
 gboolean sro_get_layer 		(SRObject *obj, SRObjectLayer *layer, SRLong index);
