@@ -2288,12 +2288,30 @@ srw_elements_from_accessible (Accessible 	*accessible,
 					     toplevel_z_order);
 				if (text_chunk)
 				{
+					gboolean only_spaces = TRUE;
+					gint i, cnt;
+						
 					if (accessible == focused_accessible)
 					{
 					    text_chunk->is_focused = TRUE;
 					}
 					offset = text_chunk->end_offset + 1;
-					if (text_chunk->string)
+					
+					/* WORKAROUND for bug 144404:
+					   a line of only spaces should be ignored 
+					   (treated as an empty line) */
+					if (text_chunk->string != NULL)
+					{
+					    cnt = strlen (text_chunk->string);   
+					    for (i = 0; i < cnt; i++)
+						if ((guint16)*(text_chunk->string + i) != 0x20 &&
+					    	    (guint16)*(text_chunk->string + i) != 0x160)   
+						{
+						    only_spaces = FALSE;
+						    break;
+						}
+					}	
+					if (text_chunk->string && !only_spaces)
 					{
 						id++;
 						text_chunk->id 		= id;
